/*
    FreeRTOS V7.0.0 - Copyright (C) 2011 Real Time Engineers Ltd.


	FreeRTOS supports many tools and architectures. V7.0.0 is sponsored by:
	Atollic AB - Atollic provides professional embedded systems development
	tools for C/C++ development, code analysis and test automation.
	See http://www.atollic.com


    ***************************************************************************
     *                                                                       *
     *    FreeRTOS tutorial books are available in pdf and paperback.        *
     *    Complete, revised, and edited pdf reference manuals are also       *
     *    available.                                                         *
     *                                                                       *
     *    Purchasing FreeRTOS documentation will not only help you, by       *
     *    ensuring you get running as quickly as possible and with an        *
     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
     *    the FreeRTOS project to continue with its mission of providing     *
     *    professional grade, cross platform, de facto standard solutions    *
     *    for microcontrollers - completely free of charge!                  *
     *                                                                       *
     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
     *                                                                       *
     *    Thank you for using FreeRTOS, and thank you for your support!      *
     *                                                                       *
    ***************************************************************************


    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    >>>NOTE<<< The modification to the GPL is included to allow you to
    distribute a combined work that includes FreeRTOS without being obliged to
    provide the source code for proprietary components outside of the FreeRTOS
    kernel.  FreeRTOS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* 
	BASIC INTERRUPT DRIVEN SERIAL PORT DRIVER FOR UART
*/

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

/* Demo application includes. */
#include "serial.h"

/* Microblaze driver includes. */
#include <xio.h>
#include "xuartlite.h"
#include "xuartlite_l.h"

/*-----------------------------------------------------------*/

static void prvReceiveHandler( void *pvCallBackReference, unsigned int iByteCount );
static void prvTransmitHandler( void *pvCallBackReference, unsigned int iByteCount );
/*-----------------------------------------------------------*/

/* Device Configuration. */
static XUartLite xUART;

/* Queues used to hold received characters, and characters waiting to be
transmitted. */
static xQueueHandle xRxedChars; 
static xQueueHandle xCharsForTx; 
/*-----------------------------------------------------------*/

xComPortHandle xSerialPortInitMinimal( unsigned long ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
{
	/* NOTE: The baud rate used by this driver is determined by the hardware
	parameterization of the UART Lite peripheral, and the baud value passed to
	this function has no effect. */

	/* Initialise the device. */
	XUartLite_Initialize( &xUART, configUART_DEVICE_ID );

	/* Create the queues used to hold Rx and Tx characters. */
	xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed char ) );
	xCharsForTx = xQueueCreate( uxQueueLength + 1, ( unsigned portBASE_TYPE ) sizeof( signed char ) );

	if( ( xRxedChars ) && ( xCharsForTx ) )
	{
		/* Disable the interrupt. */
		XUartLite_DisableInterrupt( &xUART );
		
		/* Flush the fifos. */
		XUartLite_ResetFifos( &xUART );

		/* Register functions that are called from the Rx/Tx interrupts. */
		XUartLite_SetRecvHandler( &xUART, prvReceiveHandler, xRxedChars );
		XUartLite_SetSendHandler( &xUART, prvTransmitHandler, xCharsForTx );

		/* "Connect" the interrupt handler. */
		xPortInstallInterruptHandler( configUART_INTR_ID, (XInterruptHandler)XUartLite_InterruptHandler, &xUART );

		/* Enable the interrupt again.  The interrupt controller has not yet been 
		initialised so there is no chance of receiving an interrupt until the 
		scheduler has been started. */
		XUartLite_EnableInterrupt( &xUART );
	}
	
	return ( xComPortHandle ) 0;
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialGetChar( xComPortHandle pxPort, signed char *pcRxedChar, portTickType xBlockTime )
{
portBASE_TYPE xReturn = pdFALSE;
	/* The port handle is not required as this driver only supports one UART. */
	( void ) pxPort;

	/* Get the next character from the buffer.  Return false if no characters
	are available, or arrive before xBlockTime expires. */
	if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
	{
		xReturn = pdTRUE;
	}
	return xReturn;
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialPutChar( xComPortHandle pxPort, signed char cOutChar, portTickType xBlockTime )
{
portBASE_TYPE xReturn = pdTRUE;

	portENTER_CRITICAL();
	{
		/* Are there things waiting to be sent? */
		if( 0 != uxQueueMessagesWaiting( xCharsForTx ) )
		{
			if( xQueueSend( xCharsForTx, &cOutChar, xBlockTime ) != pdPASS )
			{
				xReturn = pdFAIL;
			}
		}
		/* Is the last character currently being sent? */
		else if ( XUartLite_IsSending( &xUART ) )
		{
			if( xQueueSend( xCharsForTx, &cOutChar, xBlockTime ) != pdPASS )
			{
				xReturn = pdFAIL;
			}
		}
		/* Otherwise, this the first character to be sent, so do it immediately. */
		else
		{
			xUART.SendBuffer.RequestedBytes = 1;
			XUartLite_SendByte( xUART.RegBaseAddress, cOutChar );
		}
	}
	portEXIT_CRITICAL();

	return xReturn;
}
/*-----------------------------------------------------------*/

void vSerialClose( xComPortHandle xPort )
{
	/* Not supported as not required by the demo application. */
	( void ) xPort;
}
/*-----------------------------------------------------------*/

void vSerialPutString( xComPortHandle pxPort, const signed char * const pcString, unsigned short usStringLength )
{
unsigned short usIndex;
	for ( usIndex = 0; usIndex < usStringLength; usIndex++ )
	{
		(void)xSerialPutChar( pxPort, pcString[ usIndex ], portMAX_DELAY );
	}
}
/*-----------------------------------------------------------*/

void prvReceiveHandler( void *pvCallBackReference, unsigned int iByteCount )
{
portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
signed char cRx;

	/* Grab the character. */
	cRx = XUartLite_RecvByte( xUART.RegBaseAddress );

	/* Place the character on the queue and call send to trigger the sending. */
	if( xQueueSendFromISR( (xQueueHandle)pvCallBackReference, &cRx, &xHigherPriorityTaskWoken ) != pdPASS )
	{
		/* Error handling. */
	}

	portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );
}
/*-----------------------------------------------------------*/

void prvTransmitHandler( void *pvCallBackReference, unsigned int iByteCount )
{
portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
signed char cTx;
	if ( pdPASS == xQueueReceiveFromISR( (xQueueHandle) pvCallBackReference, &cTx, &xHigherPriorityTaskWoken ) )
	{
		xUART.SendBuffer.RequestedBytes = 1;
		XUartLite_SendByte( xUART.RegBaseAddress, cTx );
	}
	portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );
}
/*-----------------------------------------------------------*/
